require 'omniauth-oauth2'

module OmniAuth
  module Strategies
    class Cisimple < OmniAuth::Strategies::OAuth2
      class NoAuthorizationCodeError < StandardError; end
      
      option :client_options, {
          :site => 'https://www.cisimple.com',
          :authorize_url => 'https://www.cisimple.com/login/oauth/authorize',
          :token_url => 'https://www.cisimple.com/login/oauth/access_token'
      }

      def request_phase
        super
      end
      
      option :access_token_options, {
        :param_name => 'access_token'
      }

      def authorize_params
        super.tap do |params|
          %w[scope client_options].each do |v|
            if request.params[v]
              params[v.to_sym] = request.params[v]
            end
          end
        end
      end

      uid { raw_info['uuid'].to_s }

      info do
        {
            'email' => email,
            'image' => image,
            'nickname' => nickname
        }
      end

      def raw_info
        access_token.options[:mode] = :query
        @raw_info ||= access_token.get('user').parsed
      end
      
      def access_token_options
        options.access_token_options.inject({}) { |h,(k,v)| h[k.to_sym] = v; h }
      end
      
      def build_access_token
        if access_token = request.params["access_token"]
          ::OAuth2::AccessToken.from_hash(
            client, 
            {"access_token" => access_token}.update(access_token_options)
          )
        else
          with_authorization_code! { super }.tap do |token|
            token.options.merge!(access_token_options)
          end
        end
      end
      
      def with_authorization_code!
        if request.params.key?('code')
          yield
        else
          raise NoAuthorizationCodeError, 'must pass code parameter'
        end
      end

      def email
        raw_info['email']
      end

      def image
        raw_info['avatar_url']
      end
      
      def nickname
        raw_info['nickname']
      end

      def email_access_allowed?
        options['scope'] =~ /user/
      end
    end
  end
end


